<?php

namespace Drupal\pack_upload\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\pack_upload\ArchiverFactory;
use Drupal\Core\StreamWrapper\StreamWrapperManager;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Extracts and uploads the archived file to the file system.
 */
class UploadForm extends FormBase {

  /**
   * Drupal File System service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Entity Type Manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Pack and upload Archiver service.
   *
   * @var \Drupal\pack_upload\ArchiverFactory
   */
  protected $archiverFactory;

  /**
   * The StreamWrapper manager service.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManager
   */
  protected $streamWrapperManager;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructor for \Drupal\pack_upload\Form\UploadForm class.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The Form Builder service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The Entity type manager service.
   * @param \Drupal\pack_upload\ArchiverFactory $archiver_factory
   *   The Archiver Factory.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManager $stream_wrapper_manager
   *   The StreamWrapper manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(
    FileSystemInterface $file_system,
    EntityTypeManagerInterface $entity_type_manager,
    ArchiverFactory $archiver_factory,
    StreamWrapperManager $stream_wrapper_manager,
    ConfigFactoryInterface $config_factory
    ) {
    $this->fileSystem = $file_system;
    $this->entityTypeManager = $entity_type_manager;
    $this->archiverFactory = $archiver_factory;
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
      $container->get('entity_type.manager'),
      $container->get('pack_upload.archiver_factory'),
      $container->get('stream_wrapper_manager'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @see \Drupal\Core\Form\FormInterface::getFormId()
   */
  public function getFormId() {
    return 'pack_upload_upload_form';
  }

  /**
   * {@inheritdoc}
   *
   * @see \Drupal\Core\Form\FormInterface::buildForm()
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['panel'] = [
      '#title' => $this->t('Bulk Media Uploader'),
      '#type' => 'fieldset',
    ];
    $config = $this->config('pack_upload.settings');
    $path = $this->fileBuildUri($config->get('path'));

    $form['panel']['file'] = [
      '#title' => $this->t('Upload file'),
      '#type' => 'managed_file',
      '#required' => TRUE,
      '#upload_location' => $path,
      '#upload_validators' => [
        'file_validate_extensions' => ['zip tar tar.gz'],
      ],
      '#description' => $this->t('Create package of media files, for e.g., PDFs, images, text files and upload to Drupal. Valid extensions are .zip, .tar.gz, .tar. All files will be extracted to :directory', [':directory' => $path]),
    ];

    $form['panel']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * @see \Drupal\Core\Form\FormInterface::submitForm()
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $uri = $this->fileBuildUri($this->config('pack_upload.settings')->get('path'));
    $uploaded_file = $form_state->getValue('file');

    /** @var \Drupal\file\FileStorageInterface $file_storage */
    $file_storage = $this->entityTypeManager->getStorage('file');
    if ($file = $file_storage->load($uploaded_file[0])) {
      $archived_file_path = $this->fileSystem->realpath($file->getFileUri());
      $extract_dir = $this->fileSystem->realpath($uri);

      // Load the appropriate archiver and extarct the archive.
      $archiver = $this->archiverFactory->getArchiver($archived_file_path);
      $result = $archiver->extractTo($extract_dir);

      if ($result === TRUE) {
        $this->messenger()->addStatus($this->t('All media have been extracted to %path',
          ['%path' => $extract_dir]));
        // Delete the archive file that was uploaded.
        $file_storage->delete([$file]);
      }
      else {
        $this->messenger()->addError($this->t('There is some problem related to extraction
          of the file. Please upload and try again.'), FALSE);
      }
    }
    else {
      $this->messenger()->addError($this->t("No file was uploaded."), FALSE);
    }
  }

  /**
   * Constructs a URI to Drupal's default files location given a relative path.
   *
   * @param string $path
   *   The relative path.
   *
   * @return string
   *   Uri to Drupal's default files location.
   */
  public function fileBuildUri($path): string {
    $uri = $this->configFactory->get('system.file')->get('default_scheme') . '://' . $path;
    return $this->streamWrapperManager->normalizeUri($uri);
  }

}
